//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "header.h"
#include "mainapp.h"
#include "mpu680x0.h"
#include "pluto.h"
#include "scheduler.h"
#include "vm.h"

void cli_halt_callback(void *);

// wxApp と相似にするためのクラス
class CLIApp
{
 public:
	int OnInit();
	bool Parse();
	int OnCreate();

	int ac;
	char **av;
};

int
main(int ac, char *av[])
{
	std::unique_ptr<CLIApp> app(new CLIApp());

	// wxApp とよく似せるため
	app->ac = ac;
	app->av = av;
	return app->OnInit();
}

// ここがエントリポイント(のつもり)
int
CLIApp::OnInit()
{
	int rv;

	// CLI 側では関係ないが wx 側で必要なので初期化は2ステージに分かれている
	rv = gMainApp.Init1(true, ac, av);
	if (rv != MainApp::PASS) {
		return rv;
	}
	if (!gMainApp.Init2()) {
		return EXIT_FAILURE;
	}

	// VM にコールバックを登録
	if (gMPU680x0) {
		gMPU680x0->SetHaltCallback(cli_halt_callback, NULL);
	}

	// メインウィンドウに相当するところ
	OnCreate();

	return EXIT_SUCCESS;
}

// メインウィンドウ作成後のイベントに相当するところ
int
CLIApp::OnCreate()
{
	// 電源投入
	gVM->PowerButton();

	// スケジューラの終了を待つ

	gScheduler->Wait();

	return 0;
}

// m680x0 ダブルバスフォールト時に呼ばれるコールバック。
void
cli_halt_callback(void *dummy)
{
	// どうする?
	warnx("Double bus fault has occurred in VM");
}
