//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

// 逆アセンブラ(共通部分)

#pragma once

#include "debugger_private.h"

// 逆アセンブラ抽象クラス
class Disasm
{
 public:
	Disasm(DebuggerMemoryStream& mem_)
		: mem(mem_)
	{
	}
	virtual ~Disasm();

	// mem.laddr の位置を1命令逆アセンブルする。
	// 成功すれば true、失敗すれば false を返す?
	// 成功すれば text に出力文字列、ir に命令バイナリがセットされている。
	virtual bool Exec() = 0;

	// 出力文字列(ダンプ)
	std::string dump {};

	// 出力文字列(命令)
	std::string text {};

	// この命令のバイナリ列
	// (メモリイメージ順)
	std::vector<uint8> bin {};

 protected:
	// メモリアクセス。
	DebuggerMemoryStream& mem;

	// 継承先の Exec() の先頭で呼び出してください。
	void ExecInit() {
		text.clear();
		bin.clear();
	}
};
