<!--
 nono
 Copyright (C) 2020 nono project
 Licensed under nono-license.txt
-->
<html><?php
	$nono_ver = "";

	if ($_SERVER['argc'] > 1)
		get_version($_SERVER['argv'][1]);

function get_version($filename)
{
	global $nono_ver;

	$fp = fopen($filename, "r");
	if ($fp !== false) {
		while (($buf = fgets($fp)) !== false) {
			if (preg_match("/NONO_MAJOR_VER\s+\((\d+)\)/", $buf, $m)) {
				$nono_major = $m[1];
			}
			if (preg_match("/NONO_MINOR_VER\s+\((\d+)\)/", $buf, $m)) {
				$nono_minor = $m[1];
			}
			if (preg_match("/NONO_PATCH_VER\s+\((\d+)\)/", $buf, $m)) {
				$nono_patch = $m[1];
			}
			if (preg_match("/NONO_DATE\s+\"([^\"]+)\"/", $buf, $m)) {
				$nono_date = $m[1];
			}
		}
		fclose($fp);

		$nono_ver = "${nono_major}.${nono_minor}.${nono_patch} (${nono_date})";
	}
}
?>
<?php require_once "common.php"; ?>
<body>
<h3>nono <?=$nono_ver?></h3>

nono は NetBSD とかで動作する OMRON LUNA-I や LUNA88K とかのエミュレータです。
<q>nono is OMRON LUNA-I and LUNA88K emulator runs on NetBSD and etc.</q>

<hr>

<h4>Index of this page:</h4>
<div class="main">
<a href="#migrate">ver 0.1.x から ver 0.2.0 への移行方法 <q>How to migrate from ver 0.1.x to ver 0.2.0</q></a><br>
<a href="#build">ビルド方法 <q>How to build</q></a><br>
<a href="#commandline">コマンドラインオプション <q>Command Line Option</q></a><br>
<a href="#configuration">設定 <q>Configuration</q></a><br>
<a href="#aboutvm">VM について <q>About VM</q></a><br>
<a href="#tryit">実行してみる <q>Try it</q></a><br>
<a href="#network">ネットワーク設定例 <q>Example of network setup</q></a><br>
<a href="#changes">変更履歴 <q>Changes</q></a><br>
<a href="#license">連絡先、ライセンス等 <q>Contact, License, etc</q></a><br>
</div>

<a name="migrate"></a>
<h4>ver 0.1.x から ver 0.2.0 への移行方法 <q>How to migrate from ver 0.1.x to ver 0.2.0</q></h4>
<div class="main">
ver 0.1.x と ver 0.2.0 には2つの非互換があり、そのままでは以前と同じ動作はしません。
<q>There are two incompatibilities between ver 0.1.x and ver 0.2.0,
so it will not work the same as before.</q>

<dl>
<dt>1. 設定ファイルのネットワーク関連項目の変更
<q>Configuration changes about network</q>
<dd>ver 0.1.x の以下の2つの設定項目は廃止になりました。
<q>These two items in 0.1.x have been removed.</q>
<br>
<blockquote style="background-color: var(--light-pink);"><pre>
ethernet-hostdriver     = (default: none)
ethernet-hostdevice     = (default: auto)
</pre></blockquote>
代わりに次の5つが追加されています。
設定方法は<a href="#configuration">このドキュメントの設定の章</a>を参照してください。
ホストドライバは以前はデフォルトでオフでしたがデフォルトでオン(auto)に変更になっています。
<q>Instead, the following five items have been added.
See <a href="#configuration">Configuration section in this document</a>
to setup.
The host driver used to be disabled by default before,
but now it is enabled(auto) by default.</q>
<blockquote style="background-color: white;"><pre>
hostnet-driver          = (default: auto)
hostnet-afpacket-ifname = (default: auto)
hostnet-bpf-ifname      = (defualt: auto)
hostnet-tap-devpath     = (default: auto)
hostnet-fallback        = (default: 0)
</pre></blockquote>
</dd>

<dt>2. LUNA88K の NVRAM サポートについて <q>About NVRAM in LUNA88K</q>
<dd>
LUNA88K では本来 PROM が NVRAM の情報を元に起動先を決定しますが、
nono ver 0.1.x は NVRAM を (ファイルを作るだけで) 一切サポートしておらず、
OpenBSD/luna88k を起動するために起動先ファイル名を boot と決め打ちしていました。
ver 0.2.0 で NVRAM をサポートしたため、
内蔵互換 ROM で起動するとまず NVRAM がチェックサム不正のため初期化されます。
更に初期化された NVRAM の起動ファイル名は vmunix であるため、
そのままでは OpenBSD/luna88k を起動できません。
Emulated ROM Monitor が起動したら以下のように入力してください。
<q>On LUNA88K, originally PROM determines the boot target by referring NVRAM.
Since nono ver 0.1.x didn't support NVRAM (though it created NVRAM.DAT file),
it used hard-coded boot filename "boot" in order to boot OpenBSD/luna88k.
nono ver 0.2.0 supports NVRAM, so at first it initializes the NVRAM
due to checksum mismatch.
In addition, the boot filename in the initialized NVRAM is "vmunix",
so it cannot boot OpenBSD/luna88k.
To boot it, enter as following on Emulated ROM Monitor.</q>
<blockquote><pre>
nvram boot_filename boot[Enter]
y
b[Enter]
</pre></blockquote>
この内容は NVRAM.DAT に記録されるので次回以降は直接 OpenBSD が起動します。
<q>The information you have just entered is recorded in the NVRAM,
so next time it boots OpenBSD automatically.</q>
</div>

<a name="build"></a>
<h4>ビルド方法 <q>How to build</q></h4>
<div class="main">
ビルドには以下が必要です。
NetBSD(pkgsrc) なら wxWidgets は pkgsrc/x11/wxGTK30 です
(OPTIONS:gtk3 で動作確認しています)。
<q>The followings are required for build.
If you use NetBSD(pkgsrc),
wxWidgets is pkgsrc/x11/wxGTK30 (OPTIONS:gtk3 is tested).</q>

<ul>
<li>make (BSD make, not GNU make)
<li>C/C++ compiler which supports -std=c++14.
<br>
(For gcc, 7.4 works but 5.5 doesn't work, at least.
For clang, 7.0 and 8.0 works at least.)
<li>wxWidgets &gt;= 3.0
<li>gettext
</ul>

<p>
(NetBSD 以外でのビルドはサポートしていませんが)
Ubuntu 18.04 ではたぶん以下のパッケージが必要です。
<q>You may need the following packages on Ubuntu 18.04
(though we won't support non-NetBSD platform).</q>
</p>
<ul>
<li>
bmake
build-essential
gettext
libbsd-dev
libedit-dev
libkqueue-dev
libwxgtk3.0-dev
zlib1g-dev
</ul>

<p>
nono のソースアーカイブを展開したら以下のようにビルドします。
configure には環境変数 CC、CXX でコンパイラを指定することが出来ます。
また wx-config が標準的な名前で提供されていないために見付けられない場合には
環境変数 WX_CONFIG にパスを指定することが出来ます。
<q>Extract the nono's source archive and build as following.
You can specify C/C++ compiler using environment variable CC and CXX
if configure cannot find standard name suitable compiler.
Also, you can specify wx-config path using environment variable WX_CONFIG
if configure cannot find wx-config.</q>
<blockquote class="cons"><pre>
% ./configure
% make depend
% make
% su
# make install
</pre></blockquote>

<p>
2つの実行ファイルがインストールされます。
<tt>nono</tt> が GUI 版実行ファイル、
<tt>nono-cli</tt> がコマンドライン版です。
<q>Two executables will be installed.
<tt>nono</tt> is the GUI executable and <tt>nono-cli</tt> is
the command line executable.</q>

</div>


<a name="commandline"></a>
<h4>コマンドラインオプション <q>Command Line Option</q></h4>
<div class="main">
<dl>
<?php
function item($name, $desc)
{
	if (!is_array($name)) {
		$name = array($name);
	}
	foreach ($name as $n) {
		print "<dt class=dt-indent><tt>{$n}</tt></dt>\n";
	}
	print "<dd>{$desc}</p></dd>\n";
}
function option_item($name, $desc)
{
	item($name, $desc);
}

option_item("-A <i>file</i>", <<<_EOM_
ホストの <tt><i>file</i></tt> をロードして実行します。
<tt><i>file</i></tt> が相対パスの場合カレントディレクトリからのパスになります。
ファイルが gzip 圧縮されていれば自動的に展開します。
(展開後の) ファイル形式は a.out (OMAGIC) か ELF で、
実際にはブートローダとカーネル程度しか想定していません。
設定ファイルの <tt>prom-image</tt> とともに指定されると
<tt>-A</tt> のほうが優先します。
<q>Loads and executes host's <tt><i>file</i></tt>.
If <tt><i>file</i></tt> is relative path,
it is path from the current directory.
If the file is gzip'd, it is automatically extracted.
The supported file format (after extracting) is a.out (OMAGIC) or ELF.
Actually, it only assumes bootloaders or kernels.
If this option is specified at the same time as
<tt>prom-image</tt> in configuration file,
this option preceeds.
</q>
_EOM_
);

option_item("-c <i>vmpath</i>", <<<_EOM_
VM ディレクトリ/設定ファイルを指定します。
<i>vmpath</i> がディレクトリならそのディレクトリの中の nono.cfg
を設定ファイルとします。
<i>vmpath</i> がファイルならそれを設定ファイルとします。
そしていずれの場合も
設定ファイルがあるディレクトリを VM ディレクトリとします。
-c オプションを省略すると <i>vmpath</i> をカレントディレクトリとします。
<q>Specifies the VM directory/configuration file.
If <i>vmpath</i> is a directory,
make nono.cfg in that directory a configuration file.
Or if <i>vmpath</i> is a file,
make the specified file a configuration file.
And in both cases,
make the directory where that file is located a VM directory.
If <tt>-c</tt> option is omitted,
<i>vmpath</i> is considered as the current directory.
</q>
_EOM_
);

option_item("-f", <<<_EOM_
高速モードで起動します。
GUI なら起動後にもメニューから変更できますが、その初期値を変えるだけです。
<q>Boot as the fast mode.
You can change this mode on GUI menu after boot,
and the option only changes its initial state.
</q>
_EOM_
);

option_item("--fontsize <i>height</i>", <<<_EOM_
GUI 版のみ。
全サブウインドウの起動時のフォントサイズを指定します。
<tt><i>height</i></tt> には 12, 16
のいずれかを指定します。デフォルトは <tt>12</tt> です。
起動後にメニューから変更できます。
<q>
GUI Only.
Specifies the initial fontsize on all sub windows.
<tt><i>height</i></tt> is one of 12 or 16.
The default value is <tt>12</tt>.
You can change this value on GUI menu after boot.
</q>
_EOM_
);

option_item(array("-s <i>scale</i>",
	"--scale <i>scale</i>"), <<<_EOM_
GUI 版のみ。
メインウィンドウの起動時のスケールを実数で指定します。
起動後にもメニューからプリセットされた倍率には変更可能ですが、
任意倍率は起動時のみ指定可能です
<span class=strike>(そのうちなんとかしたい)</span>。
デフォルトは 1.0 です。
<q>
GUI Only.
Specifies the initial main window scale in real number.
You can change this scale on GUI menu after boot,
but unlike this option, there are only a few preset choices
<span class=strike>(Should be improved in someday)</span>.
The default value is 1.0</q>
_EOM_
);

option_item("--show-config", <<<_EOM_
設定ファイルと <tt>-V</tt> オプションを読み込んだ結果を表示します。
<q>Shows the result of reading configuration file and
parsing <tt>-V</tt> options.</q>
_EOM_
);

option_item("-v", <<<_EOM_
バージョンを表示します。
<q>Shows the version.</q>
_EOM_
);

option_item("-V <i>name</i>=<i>value</i>", <<<_EOM_
設定ファイルで指定した <tt><i>name</i>=<i>configvalue</i></tt> の代わりに
このオプションの <tt><i>name</i>=<i>value</i></tt> を適用します。
<q>Use this <i>name</i>=<i>value</i>
instead of <i>name</i>=<i>configvalue</i> specified in configuration file.
</q>
_EOM_
);
?>
</dl>

以下開発用。<q>For developers:</q>
<dl>
<?php
// 項目間を空けない
function option_dev($name, $desc)
{
	print "<dt class=dt-indent><tt>{$name}</tt></dt>\n";
	print "<dd>{$desc}</dd>\n";
}

option_dev("-b <i>hexaddr</i>[,<i>skipcount</i>]", <<<_EOM_
デバッガのブレークポイントを 16進数で指定します。
_EOM_
);

option_dev("-C", <<<_EOM_
ログをコンソールにも出力します。
通常はログウィンドウにだけ出力されます。
_EOM_
);

option_dev("-d", <<<_EOM_
起動時にデバッガプロンプトで停止します。
_EOM_
);

option_dev("-D", <<<_EOM_
コンソールをデバッガとして使用します。
-d を指定しなくても起動時にプロンプトで停止します
<span class=strike>(そのうちなんとかしたい)</span>。
_EOM_
);

option_dev("-L <i>name1</i>=<i>level1</i>[,<i>name2</i>=<i>level2</i>,...]",
<<<_EOM_
ログレベルを指定します。
カンマで区切って複数指定することも出来ます。
<tt>-Lhelp</tt> で name の一覧を表示します。
_EOM_
);

option_dev("-M <i>name</i>[,<i>name2</i>,...]", <<<_EOM_
起動時に表示するモニタウィンドウを指定します。
カンマで区切って複数指定することも出来ます。
<tt>-Mhelp</tt> で name の一覧を表示します。
_EOM_
);


?>
</dl>
</div>


<a name="configuration"></a>
<h4>設定 <q>Configuration</q></h4>
<div class="main">
VM の設定ファイルはその VM ディレクトリ内の nono.cfg
(または -c で指定したファイル) です。
書式は <tt>key = value</tt> 形式で1行1項目ずつです。
<tt>key</tt> と <tt>value</tt> の前後の空白は取り除かれます。
また空行と "<tt>#</tt>" で始まる行は無視します。
知らないキーの行も無視します。
<q>VM configuration file is nono.cfg
(or the file specified by -c option) in the VM directory.
Its syntax is <tt>key = value</tt> format, one per line.
White spaces before and after <tt>key</tt> and <tt>value</tt> are ignored.
And, blank lines, lines beginning with "<tt>#</tt>", and
lines with unrecognized key are also ignored.
</q>
<p>
設定項目は次の通りです。
<q>The configuration items are:</q>
<dl>
<?php
function config_item($name, $desc)
{
	item($name, $desc);
}

config_item("vmtype = <i>string</i>", <<<_EOM_
VM 種別を以下のいずれかから指定します。
省略不可です。
<q>Specifies the VM type from the following.
This field is mandatory.
</q>
<table cellspacing=0 cellpadding=0>
<tr><td>&nbsp;<td><tt>luna</tt>	<td>… LUNA-I
<tr><td><td><tt>luna88k</tt>	<td>… LUNA88K
</table>
_EOM_
);

config_item("clock-sync = <i>value</i>", <<<_EOM_
仮想マシン内の時刻の同期方法を指定します。
<tt>real</tt> なら実時間に同期、<tt>virtual</tt> なら仮想時間に同期します。
デフォルトは <tt>real</tt> です。
この機能は実験中のため将来予告なく仕様が変更になる可能性があります。
<q>Specify how to synchronize the time in virtual machine.
If <tt>real</tt>, synchronize with the real time;
if <tt>virtual</tt>, synchronize with the virtual time.
The default is <tt>real</tt>.
This feature is under experimentation and may be changed in the future
without notice.
</q>
_EOM_
);

config_item("debugger-port = <i>integer</i>", <<<_EOM_
デバッガの TCP 待ち受けポート番号を指定します。
0 なら待ち受けを行いません。
デフォルトは 0 です。
<q>Specifies the TCP port number that debugger listens.
If 0, it will not listen.
The default is 0.</q>
_EOM_
);

config_item("ethernet-macaddr = <i>string</i>", <<<_EOM_
イーサネットデバイスの仮想マシン側の MAC アドレスを指定します。
<tt><i>XX</i>:<i>XX</i>:<i>XX</i>:<i>XX</i>:<i>XX</i>:<i>XX</i></tt>
形式で指定します。
<tt>auto</tt> なら自動的に決定します。
デフォルトは <tt>auto</tt> です。
_EOM_
);

config_item("hostnet-driver = <i>string</i>", <<<_EOM_
イーサネットデバイスのホスト側ドライバを指定します。
<tt>afpacket</tt>、<tt>bpf</tt>、<tt>tap</tt> のうち
ホスト OS がサポートしているものと <tt>auto</tt>、<tt>none</tt> が選択可能です。
<tt>afpacket</tt> は Linux の AF_PACKET ソケットを使用します。
<tt>bpf</tt> は bpf(4) デバイスを使用します。
<tt>tap</tt> は tap(4) インタフェースを使用します。
<tt>none</tt> ならホスト側とは一切通信を行いません。
<tt>auto</tt> ならホスト OS がサポートしているもののうち
<tt>tap</tt> → <tt>afpacket</tt> → <tt>bpf</tt> を順に試します。
デフォルトは <tt>auto</tt> です。
<q>Specifies the host driver of ethernet device.
<tt>afpacket</tt>, <tt>bpf</tt>, and <tt>tap</tt> can be specified
only if the host OS supports them.
<tt>auto</tt> and <tt>none</tt> can always be specified.
<tt>afpacket</tt> uses Linux's AF_PACKET socket,
<tt>bpf</tt> uses bpf(4) device,
and <tt>tap</tt> uses tap(4) interface.
<tt>none</tt> doesn't make any communication with the host.
If <tt>auto</tt> is specified,
it tries <tt>tap</tt>, <tt>afpacket</tt>, and <tt>bpf</tt> in that order
(if the host OS supports them).
The default is <tt>auto</tt>.</q>
_EOM_
);

config_item("hostdriver-afpacket-ifname = <i>ifname</i>", <<<_EOM_
ホストドライバが afpacket の時にバインドするインタフェースを1つ指定します。
<i>ifname</i> が <tt>auto</tt>
なら使用可能なインタフェースを1つ自動的に選択します。
デフォルトは <tt>auto</tt> です。
なお、このホストドライバではローカルホストとの通信は出来ません。
<q>Specify an interface name to bind to, when the host driver is afpacket.
If <tt>auto</tt> is specified as <i>ifname</i>,
it selects an usable interface automatically.
The default is <tt>auto</tt>.
Note that this host driver cannot communicate with the localhost.</q>
_EOM_
);

config_item("hostdriver-bpf-ifname = <i>ifname</i>", <<<_EOM_
ホストドライバが bpf の時にバインドするインタフェースを1つ指定します。
<i>ifname</i> が <tt>auto</tt>
なら使用可能なインタフェースを1つ自動的に選択します。
デフォルトは <tt>auto</tt> です。
なお、このホストドライバではローカルホストとの通信は出来ません。
<q>Specify an interface name to bind to, when the host driver is bpf.
If <tt>auto</tt> is specified as <i>ifname</i>,
it selects an usable interface automatically.
The default is <tt>auto</tt>.
Note that this host driver cannot communicate with the localhost.</q>
_EOM_
);

config_item("hostdriver-tap-devpath = <i>path</i>", <<<_EOM_
ホストドライバが tap の時に使用するデバイスを1つフルパスで指定します。
デフォルトは <tt>auto</tt> で、この場合は自動的にデバイスを選択します。
この時の探し方はホスト OS によって異なります。
Linux なら <tt>/dev/net/tun</tt> を指定したのと同じです。
OpenBSD なら <tt>/dev/tap0</tt> から <tt>/dev/tap9</tt> まで
順番にオープンできるまで試します。
NetBSD (と FreeBSD) ならまず <tt>/dev/tap</tt> でクローニングを試み、
それが失敗すれば <tt>/dev/tap0</tt> から <tt>/dev/tap9</tt> までを
順番にオープンできるまで試します。
いずれの場合も VM ディレクトリかその親ディレクトリに
<tt>nono-ifup</tt>, <tt>nono-ifdown</tt> という名前のシェルスクリプトが必要です。
何もすることがない場合でも正常終了する空のシェルスクリプトを用意してください。
<q>
Specify a device pathname, when the host driver is tap.
The default is <tt>auto</tt>.
The behavior in this case depends on the host OS.
On Linux, it's the same as <tt>/dev/net/tun</tt>.
On OpenBSD, it will try from <tt>/dev/tap0</tt> to <tt>/dev/tap9</tt>
until successful.
On NetBSD (and FreeBSD), it will try cloning by <tt>/dev/tap</tt> first.
If that fails, then try from <tt>/dev/tap0</tt> to <tt>/dev/tap9</tt>
until successful.
In all cases, you need to prepare two shell script files which names are
<tt>nono-ifup</tt> and <tt>nono-ifdown</tt> in the VM directory or
its parent directory.
Even if you don't have anything to do in these scripts,
you need to prepare empty scripts that will terminate successfully.
</q>
_EOM_
);

config_item("hostnet-fallback = <i>integer</i>", <<<_EOM_
<tt>hostnet-driver</tt> で指定したドライバが使用可能でなかった時、
0 ならプロセスを終了します。
1 なら none を選択して実行を継続します。
デフォルトは 0 です。
<q>Specify the behavior when the driver which is selected by
<tt>hostnet-driver</tt> is unusable;
terminate the process if 0,
or continue to run using the <tt>none</tt> driver if 1.
The default is 0.</q>
_EOM_
);

config_item("luna-dipsw1 = <i>string</i>", <<<_EOM_
本体前面 DIPSW#1-1..#1-8 の内容を指定します。
"<tt>0</tt>" を DOWN、"<tt>1</tt>" を UP として、
これを8つ並べた形式で、前から順に #1..#8 に対応します。
<q>Specifies status of the front panel DIPSW#1-1..#1-8 using 8 digit.
"<tt>0</tt>" means DOWN and "<tt>1</tt>" means UP.
The first character corresponds to #1 and
the eighth character corresponds to #8.</q>
<p>
LUNA-I でのデフォルトは <tt>11110111</tt> です。
各スイッチの内容は以下のリンクを参照してください。
<q>On LUNA-I, the default value is <tt>11110111</tt>.
See the following link about DIPSW.</q><br>
→ <a href="https://wiki.netbsd.org/ports/luna68k/luna68k_info/"
>NetBSD/luna68k: Information</a>
<br>
LUNA88K でのデフォルトは <tt>11111111</tt> です。
各スイッチの内容は以下のリンクを参照してください。
<q>On LUNA88K, the default value is <tt>11111111</tt>.
See the following link about DIPSW.</q><br>
→ <a href="http://man.openbsd.org/boot_luna88k.8"
>OpenBSD manual pages: boot_luna88k(8)</a>
_EOM_
);

config_item("luna-dipsw2 = <i>string</i>", <<<_EOM_
本体前面 DIPSW#2-1..#2-8 の内容を指定します。
書式は <tt>luna-dipsw1</tt> と同じです。
デフォルトは <tt>11111111</tt> です。
<q>Specifies status of the front panel DIPSW#2-1..#2-8.
The same syntax as <tt>luna-dipsw1</tt> is used.
The default value is <tt>11111111</tt>.</q>
<p>
NetBSD/luna68k のブートローダは、
DIPSW#2 が "<tt>11111111</tt>" なら自動的にカーネルをロードして実行し、
どれかでも "<tt>0</tt>" にするとプロンプトで停止するようです。
<span class=strike>(本当は #8 だけで制御するつもりだったんじゃないかという気がします)</span>
<q>NetBSD/luna68k bootloader will automatically load and execute the kernel,
if the DIPSW#2 is "<tt>11111111</tt>".
Otherwise, the bootloader will enter interactive mode.
<span class=strike>(I doubt that they actually wanted to switch with only #8)
</span>
</q>
_EOM_
);

config_item("monitor-rate = <i>integer</i>", <<<_EOM_
テキスト系モニタウィンドウの更新頻度を Hz 単位で指定します。
1 から 60 までの間で指定でき、デフォルトは 20Hz です。
起動後にメニューからプリセットされた頻度には変更可能です。
<q>Specifies refresh rate of all text monitor windows in Hz.
It ranges from 1 to 60.  The default is 20Hz.
You can change this value on GUI menu after boot,
but unlike this configuration value, there are only a few preset choices.
</q>
_EOM_
);

config_item("mpu-clock = <i>value</i>", <<<_EOM_
MPU のクロック数を MHz 単位で指定します。
デフォルトは LUNA-I なら 20MHz、LUNA88K なら 25MHz です。
<q>Specifies the MPU clock in MHz.
The default value is 20MHz on LUNA-I, or 25MHz on LUNA88K.</q>
_EOM_
);

/* luna には不要
  config_item("mpu-has-fpu", ""); */

config_item("mpu-pseudo-stop = <i>integer</i>", <<<_EOM_
m88100 にて疑似 STOP 状態を有効にするかどうかを指定します。
0 なら無効(実機と同じ動作)、1 なら有効で、デフォルトは 1 です。
m88100 には、m68k の STOP 命令 (割り込みが上がるまで何もせず待つ)
に相当する命令がなく、
大抵ビジーウェイトループで割り込みが上がるのを待つことになります。
これは実機では (消費電力を減らす手段がないという些細な問題以外には)
何のデメリットもないのですが、
エミュレータで特に高速動作させている時には割り込みが上がるまで
(例えば人間がキーを入力するまで) ホスト CPU パワーを使い潰してビジーウェイトループを実行し続けることになり、ホスト CPU があっつあつになります。
それを防ぐための機能です。
特徴的な命令列を検出して実現しているので、すべての状況で動作するわけではありません。
_EOM_
);

config_item("prom-image = <i>path</i>", <<<_EOM_
LUNA-I/LUNA88K の外部 ROM イメージファイルのパスを指定します。
<i>path</i> がファイル名のみなら VM ディレクトリとその親ディレクトリからこのファイル名を検索します。
<i>path</i> が相対パスなら VM ディレクトリからの相対パスになります (現在のディレクトリからではありません)。
空にすると内蔵 ROM を使用します。
デフォルトは空です。
<q>Specifies the LUNA-I/LUNA88K's external ROM image file path.
If the <i>path</i> does not have any path delimiters,
the VM directory and then its parent directory will be searched.
If the <i>path</i> is a relative path,
it will be path from the VM directory, not from the current
directory.
If the <i>path</i> is empty, internal emulated ROM will be used.
The default value is empty.</q>
<p>
実機を持っていない場合はこの値を空に (= デフォルトのままに) しておくと、
nono 内蔵のなんちゃって下位互換 ROM で起動します。
<q>If you does not have the real LUNA machines,
you can boot with nono's internal downward compatible emulated ROM
if you set this field empty (or leave it as the default).
</q>
<p>
LUNA-I 実機を持っている場合は
ROM ファイルを指定することで実機 ROM で起動できます。
ROM ファイルは実機の 0x41000000-0x4101ffff (128KB) を保存したものです。
今のところ ROM は V4.22 (Thu Jul 27 11:45:42 1989) のみサポートしています。
それ以外については何も分かりません。
<q>If you have the real LUNA-I machine,
you can boot with the real ROM spcifying the ROM file path.
The ROM file is extracted from 0x41000000-0x4101ffff (128KB) of
the real LUNA-I machine.
For now, only V4.22 (Thu Jul 27 11:45:42 1989) is supported.
I have no idea about other ROMs.
</q>
<p>
LUNA88K 実機の場合は 0x41000000-0x4103ffff (256KB) を保存したものです。
今のところ ROM は version 1.20 のみサポートしています。
<q>
For LUNA88K,
the ROM file is extracted from 0x41000000-0x4103ffff (256KB).
For now, only version 1.20 is supported.
</q>
_EOM_
);

config_item("ram-size = <i>integer</i>", <<<_EOM_
搭載する RAM サイズを MB 単位で指定します。
LUNA-I のデフォルトは 16MB です。
16MB 未満は 4MB 単位で、
16MB 以上は 255MB まで 1MB 単位で指定できます
(ちなみに NetBSD/luna68k の起動には最低でも 8MB 必要です)。
LUNA88K のデフォルトは 64MB です。
64MB 未満は 16MB 単位で、
64MB 以上は暫定で 255MB まで 1MB 単位で指定できます。
<q>Specifies the RAM size in MB.
On LUNA-I, the default is 16MB.
If the size is less than 16MB, you can specify in 4MB unit.
If larger, you can specify up to 255MB in 1MB unit.
By the way, NetBSD/luna68k needs at least 8MB to boot.
On LUNA88K, the default is 64MB.
If the size is less than 64MB, you can specify in 16MB unit.
If larger, you can specify up to tentative 255MB in 1MB unit.
</q>
_EOM_
);

config_item("show-statuspanel = <i>integer</i>", <<<_EOM_
ステータスパネルを表示するかどうかを指定します。
0 なら非表示、1 なら表示です。
起動後はメニューから変更可能です。
<q>Specifies whether to display the status panel or not.
If 0, it is hidden; if 1, it is shown.
You can change it on GUI menu after boot.
</q>
_EOM_
);

config_item("spc0-id<i>N</i>-image = <i>devtype</i>,<i>path</i>", <<<_EOM_
SCSI デバイスを指定します。<i>N</i> には 0 から 7 が入ります。
ID 7 は本体が使用しますので指定しないでください。
値はデバイス種別 <i>devtype</i> とイメージパス <i>path</i>
を "<tt>,</tt>"(カンマ) で区切って並べた形式で、
今の所デバイス種別 <i>devtype</i> には "<tt>hd</tt>"(ハードディスク) のみ指定可能です。
ディスクイメージパスが相対パスなら VM ディレクトリからの相対パスになります。
<q>
Specifies SCSI device.  <i>N</i> is 0 to 7.
But don't specify ID 7 because the host uses it.
The value is in a form of device type <i>devtype</i> and
the image path <i>path</i> separated by "<tt>,</tt>"(comma).
For now, only "<tt>hd</tt>" (hard disk) can be specified for <i>devtype</i>.
If the <i>path</i> is relative path, it is from the VM directory.
</q>

<p>
例えば、nono.cfg と同じディレクトリに置いたディスクイメージ luna.img
を使う場合、
LUNA では通常 ID 6 をプライマリ HDD に割り当てるのでこんな感じになります。
<q>For example, if you use disk image luna.img placed in the same
directory as nono.cfg,
since LUNA usually assigns ID 6 to the primary HDD,
write as following:</q>
<blockquote><pre>
spc0-id6-image = hd,luna.img
</pre></blockquote>
_EOM_
);

config_item("spc0-id<i>N</i>-seektime = <i>integer</i>", <<<_EOM_
指定の SCSI HDD の平均シークタイムを msec 単位で指定します。
現在のデフォルトは <tt>0</tt> です (S・S・D!! S・S・D!!)。
16 程度を指定すると幾分往時に思いを馳せることが出来るかもしれませんが、
今の所あまり安定していません。
<q>Specifies the average seek time of specified SCSI HDD in msec.
Currently, the default value is <tt>0</tt>
(This may be something like SSD :-).
If you specify about 16 or so, you can feel nostalgic,
but this feature is still unstable.
</q>
_EOM_
);

config_item("spc0-id<i>N</i>-writeprotect = <i>integer</i>", <<<_EOM_
指定の SCSI デバイスへの書き込みを無視するかどうか指定します。
<tt>0</tt> なら通常動作(書き込みを行う)です。
<tt>1</tt> なら書き込みコマンドは成功したように振る舞いますが実際には
ディスクイメージに一切書き戻しません。
fsck を気にせずカーネルのデバッグとかを行いたい場合にはどうぞ。
何が起きるか意味が分からない人は指定しないでください。
デフォルトは <tt>0</tt> です。
<q>Specifies whether nono ignores writing to SCSI devices.
<tt>0</tt> means normal operation (writes to the devices).
If <tt>1</tt> is specified,
nono will not actually write back to the disk image
even though the write command is succeeded.
nono's SCSI devices acts as
write command is successfully done but it never writes back
to the actual disk image.
This is useful for kernel debugging because it does not require fsck
after the kernel hangs.
But don't use this flag if you don't understand this paragraph.
The default value is <tt>0</tt>.
</q>
_EOM_
);
?>
</dl>
</div>


<a name="aboutvm"></a>
<h4>VM について <q>About VM</q></h4>
<h5>ステータスパネル <q>Status Panel</q></h5>
<div class="main">
ステータスパネル中央にあるパフォーマンスメータのアイコンは
高速モードの状態を表示しています。
ダブルクリックすると高速/等速モードの指定を切り替えることができます。
<q>The performance meter's icon at the center of the status panel
shows the VM speed status.
You can switch full speed / syncronized mode by double-clicking on this icon.
</q>
<table>
<tr><td valign=top>
<image src="image/sp-ffmark0.png" style="padding:0.5ex; background-color:white">
<td valign=top>…
<td valign=top>
マークなしの場合、ユーザが等速モードを指定していて、等速モードで実行中です。
<q>When no icons are displayed,
the user has specified syncronized mode and
the VM is running in synchronized mode.</q>
<tr><td valign=top>
<image src="image/sp-ffmark1.png" style="padding:0.5ex; background-color:white">
<td valign=top>…
<td valign=top>
三角2つの場合、ユーザが高速モードを指定していて、等速モードで実行中です。
キー入力中(後述)または VM がアイドル状態(m68k の STOP 命令)などで起きます。
高速モード中はパーセントではなく何倍速で動作しているかを表します。
<q>When an icon (two triangles) is displayed,
the user has specified full speed mode and
the VM is running in synchronized mode.
This will happen during keystrokes (see below) or when the VM is idle
(STOP instruction on m68k).
</q>
<tr><td valign=top>
<image src="image/sp-ffmark2.png" style="padding:0.5ex; background-color:white">
<td valign=top>…
<td valign=top>
三角3つの場合、ユーザが高速モードを指定していて、高速モードで実行中です。
<q>When an icon (three triangles) is displayed,
the user has specified full speed mode and
the VM is running in full speed mode.
</table>
</div>

<h5>キー入力 <q>Key input</q></h5>
<div class="main">
LUNA では、キーボードがハードウェア側でキーリピートを行わず、
ソフトウェア(OS など)がキーリピートの処理を行います。
そのため、キーリピートを実装していない LUNA の PROM や
NetBSD/luna68k のブートローダではキーリピートは起きず、
キーリピートを実装している NetBSD/luna68k カーネルではキーリピートが起こせます。
キーリピートを起こす間隔をゲストのソフトウェアが測定しているということは、
VM が高速動作するとキーリピートもそれに合わせて発生することになり、
これをホスト側から防ぐ手段はありません。
そこで nono ではキー入力が発生している間
(キーが一つでも押されてからキーが全て離されるまでの間)、
高速モードが指示されていても VM を一時的に等速モードに落として実行します。
上記のアイコンがそれを区別しているのはこのためです。
そのため、何らかの理由でキーが入りっぱなしになった場合
(ALT + TAB やアクセラレータキーでメニューを開くと起きがちです)
高速モードが抑制されたままということが起きえます。
その場合はソフトウェアキーボードからキー入力を解除するなどしてください。
<q>
On LUNA,
key repeat is done by software(OS), not by the keyboard hardware.
For this reason,
key repeat doesn't occur on LUNA's PROM or NetBSD/luna68k's bootloader
that don't implement it, and
key repeat occurs on NetBSD/luna68k kernel that implements it.
Since the timing of key repeat is measured by the guest software,
if the VM is running faster than the real,
the key repeat will occur faster, too.
The host application doesn't have the way to avoid it.
Therefore, nono will temporarily suppress the full speed mode
while any keys are pressed.
That is why the above-mentioned icon distinguishes them.
If keys continue to be pressed for some reasons,
the VM also continues to run syncronized mode.
In this case, you can resolve it by using the software keyboard window.
</q>
</div>


<a name="tryit"></a>
<h4>実行してみる <q>Try it</q></h4>
<h5>NetBSD/luna68k を実行してみる <q>Try NetBSD/luna68k</q></h5>
<div class="main">
つついさんが NetBSD/luna68k 9.2 の liveimage を用意されています。
<q>Tsutsui-san has provided a liveimage of NetBSD/luna68k 9.2.</q>
<dl>
<dd><a href="https://twitter.com/tsutsuii/status/1262429647364427783"
>https://twitter.com/tsutsuii/status/1262429647364427783</a><br>
<dd><a href="https://twitter.com/tsutsuii/status/1405206240913805313"
>https://twitter.com/tsutsuii/status/1405206240913805313</a><br>
</dl>
ここではこれを起動してみます。<q>Let's try it.</q>
<br>
<ol>
<li>どこかに nono 用のディレクトリを用意し(例えば ~/nono/)、
その中に VM ディレクトリを用意します (例えば ~/nono/luna/)。
<q>Create a directory somewhere for nono (for example ~/nono/),
and create subdirectories for individual VMs in it
(for example ~/nono/luna/).</q>

<li>以下のリンクからイメージファイルをダウンロードして展開し、
VM ディレクトリ ~/nono/luna/ に置きます。
<q>Download imagefile from the following link,
extract it and place it in the VM directory, ~/nono/luna/.</q>
<blockquote>
<a href="http://teokurebsd.org/netbsd/liveimage/20210614-luna68k/"
>http://teokurebsd.org/netbsd/liveimage/20210614-luna68k/</a>
</blockquote>

<li>以下の内容の設定ファイル nono.cfg を VM ディレクトリ ~/nono/luna/
に作成します。
ここでは説明を簡単にするためネットワークなしにしていますが、
ネットワーク設定は必要に応じて行ってください。
<q>Create a configuration file nono.cfg in the VM directory, ~/nono/luna/,
with following contents.
By the way, to simplify the explanation, we assume there is no network here.
However, please configure the network if necessary.</q>
<blockquote class="file"><pre>
vmtype = luna
spc0-id6-image = hd,liveimage-luna68k-raw-20210614.img
hostnet-driver = none
</pre></blockquote>

<li><tt>nono -c ~/nono/luna</tt> で起動します
(VM ディレクトリに自動的に NVRAM.DAT が作られます)。
<q>Run as <tt>nono -c ~/nono/luna</tt>.
(NVRAM.DAT will be created automatically in the VM directory)</q>
<li>Emulated ROM Monitor が起動するので、
初回は以下のように入力すると NetBSD が起動します。
<q>The emulated ROM Monitor will be executed.
Then, only for the first time,
entering the following can boot NetBSD.</q>
<blockquote><pre>
k&#x23ce;
&#x23ce;
&#x23ce;
d&#x23ce;
boot&#x23ce;
g&#x23ce;
x&#x23ce;
</pre></blockquote>
画面はこんな感じのはずです (太字が入力部分)。
<q>You will see a screen like this.  The bold text indicates
the characters you need to enter.</q>
<blockquote class="file"><pre>
NONO 0.2.0 Emulated ROM Monitor for LUNA-I

** NVRAM Initialized.

&gt;<b>k&#x23ce;</b>
controller: dk  ?<b>&#x23ce;</b>
drive unit: 0  ?<b>&#x23ce;</b>
partition : c  ?<b>d&#x23ce;</b>
filename  : vmunix  ?<b>boot&#x23ce;</b>
&gt;<b>g&#x23ce;</b>
Loaded.  Entry point = $00700000
&gt;<b>x&#x23ce;</b>
</pre></blockquote>
この内容は NVRAM.DAT に記録されているので次回以降は直接 NetBSD が起動します。
<q>The information you have just entered is recorded in the NVRAM,
so next time it boots NetBSD automatically.</q>
<li>初回起動時、
Updating fontconfig cache はあほみたいに時間がかかりますが、
nono がハングアップしてるわけではありません(>_<)。
また初回ログイン時めちゃくちゃ重たいですが、
これはバックグラウンドで makemandb が動くためで
nono のせいではありません(>_<)。
<q>At the first boot, you will see the console stops after printing
"Updating fontconfig cache".
This is because the infamous fontconfig takes very looooong time.
nono would not have hang-up.
In addition, after the first login, you will feel it's too heavy.
This is because the infamous makemandb(8) runs heavily in the background
for a looooong time.
It's very sad to me that these two accidents which are far from the ideal
are the first experiences of a newcomer.
</q>

<li>終了する時は root
ユーザで &ldquo;<tt>shutdown -p now</tt>&rdquo; を実行してください。
LUNA はソフトウェアから電源オフでき、
VM の電源オフで nono も終了します。
<q>To quit, type &ldquo;<tt>shutdown -p now</tt>&rdquo; as the root user.
LUNA can be powered off by software,
and nono will terminate when the VM is powered off.</q>
</ol>
</div>

<h5>OpenBSD/luna88k を実行してみる <q>Try OpenBSD/luna88k</q></h5>
<div class="main">
あおやまさんが OpenBSD/luna88k 6.9 の liveimage を用意されています。
<q>Aoyama-san has provided a liveimage of OpenBSD/luna88k 6.9.</q>
<dl>
<dd><a href="https://twitter.com/ao_kenji/status/1404784588015112192"
>https://twitter.com/ao_kenji/status/1404784588015112192</a>
</dl>
ここではこれを起動してみます。<q>Let's try it.</q>
<br>
<ol>
<li>どこかに nono 用のディレクトリを用意し(例えば ~/nono/)、
その中に VM ディレクトリを用意します (例えば ~/nono/luna88k/)。
<q>Create a directory somewhere for nono (for example ~/nono/),
and create subdirectories for individual VMs in it
(for example ~/nono/luna88k/).</q>

<li>以下のリンクから liveimage-luna88k-raw-YYYYMMDD.img.gz
をダウンロードして展開し、
VM ディレクトリ ~/nono/luna88k/ に置きます。
<q>Download liveimage-luna88k-raw-YYYYMMDD.img.gz from the following link,
extract it and place it in the VM directory, ~/nono/luna88k/.</q>
<blockquote>
<a href="http://www.nk-home.net/~aoyama/liveimage/"
>http://www.nk-home.net/~aoyama/liveimage/</a>
</blockquote>

<li>以下の内容の設定ファイル nono.cfg を VM ディレクトリ ~/nono/luna88k/
に作成します。
ここでは説明を簡単にするためネットワークなしにしていますが、
ネットワーク設定は必要に応じて行ってください。
<q>Create a configuration file nono.cfg in the VM directory, ~/nono/luna88k/,
with following contents.
By the way, to simplify the explanation, we assume there is no network here.
However, please configure the network if necessary.</q>
<blockquote class="file"><pre>
vmtype = luna88k
spc0-id6-image = hd,liveimage-luna88k-raw-20210614.img
hostnet-driver = none
</pre></blockquote>

<li><tt>nono -c ~/nono/luna88k</tt> で起動します
(VM ディレクトリに自動的に NVRAM.DAT が作られます)。
<q>Run as <tt>nono -c ~/nono/luna88k</tt>.
(NVRAM.DAT will be created automatically in the VM directory)</q>
<li>Emulated ROM Monitor が起動するので、
初回は以下のように入力すると OpenBSD が起動します。
<q>The emulated ROM Monitor will be executed.
Then, only for the first time,
entering the following can boot OpenBSD.</q>
<blockquote><pre>
nvram boot_filename boot&#x23ce;
y
b&#x23ce;
</pre></blockquote>
画面はこんな感じのはずです (太字が入力部分)。
<q>You will see a screen like this.  The bold text indicates
the characters you need to enter.</q>
<blockquote class="file"><pre>
NONO 0.2.0 Emulated ROM Monitor for LUNA88K

** NVRAM Initialized.

N&gt;<b>nvram boot_filename boot&#x23ce;</b>
Update boot_filename : "vmunix" -&gt; "boot" (Y/[N]):<b>y</b>
Updated
N&gt;<b>b&#x23ce;</b>
</pre></blockquote>
この内容は NVRAM.DAT に記録されているので次回以降は直接 OpenBSD が起動します。
<q>The information you have just entered is recorded in the NVRAM,
so next time it boots OpenBSD automatically.</q>

<li>終了する時は root
ユーザで &ldquo;<tt>shutdown -p now</tt>&rdquo; を実行してください。
LUNA88K はソフトウェアから電源オフでき、
VM の電源オフで nono も終了します。
<q>To quit, type &ldquo;<tt>shutdown -p now</tt>&rdquo; as the root user.
LUNA88K can be powered off by software,
and nono will terminate when the VM is powered off.</q>
</ol>
</div>



<a name="network"></a>
<h4>ネットワーク設定例 <q>Example of network setup</q></h4>
<div class=main>
wm0 を持つ NetBSD ホストに tap(4) デバイスを用いて
nono のゲスト OS を接続する場合の設定例です。
<ol>
<li>
設定ファイル nono.cfg に以下の行を追加します
(と言いつつ NetBSD では書かなくてもデフォルトでこの動作になりますが)
<q>Add the following line to configuration file, nono.cfg.
(Although you don't need to write it since these are default behavior
on NetBSD)</q>
<blockquote class="file"><pre>
hostnet-driver = tap
hostnet-tap-devpath = auto
</pre></blockquote>

<li>デフォルトでは /dev/tap は一般ユーザからアクセスできないので、
chmod で適当にパーミッションを与えます。
番号の付いていないほうの /dev/tap だけでいいです。
sysinst 等で OS をアップグレードするとパーミッションが 600
に戻るのがハマりポイントです。
<q>By default, /dev/tap is only accessible to privileged user.
You need to chmod /dev/tap (without unit number) appropriately.
Note that upgrading using sysinst always reset the permission to 600.
</q>

<li>bridge(4) インタフェースを作成し、
ホストの外部(物理)インタフェースをブリッジに追加しておきます。
<q>Create a bridge(4) interface, and
add your physical interface to the bridge.</q>
<blockquote class="cons"><pre>
# ifconfig bridge0 create
# brconfig bridge0 add wm0
</pre></blockquote>
常用するなら /etc の設定ファイルに書いておきましょう。
<q>If you use it regularly,
you can put configuration file into /etc.</q>
<blockquote>
/etc/ifconfig.bridge0
<pre class=file>
create
up
!/sbin/brconfig $int add wm0
</pre>
<pre class=cons>
# /etc/rc.d/network restart
</pre>
</blockquote>

<li>
一般ユーザに戻って、
VM ディレクトリかその親ディレクトリに
次のような 2つのスクリプトを用意します。
nono は tap(4) をオープンし、
そのデバイス名を引数にこれらのスクリプトを呼びます。
sudo の設定は別途行ってください。
<q>Return to non-privileged user, and
create following two scripts in the VM directory or its parent directory.
nono will open tap(4) and
invoke these scripts with the name of the device as an argument.
In addition, you need to set up sudo separately.</q>
<blockquote>
nono-ifup
<pre class=file>
#!/bin/sh
sudo /sbin/ifconfig $1 up
sudo /sbin/brconfig bridge0 add $1
</pre></blockquote>

<blockquote>
nono-ifdown
<pre class=file>
#!/bin/sh
sudo /sbin/brconfig bridge0 delete $1
sudo /sbin/ifconfig $1 down
</pre></blockquote>

<blockquote class="cons"><pre>
% chmod +x nono-ifup nono-ifdown
</pre></blockquote>

<li>nono を起動し、
メニューの「モニタ &gt; ホスト &gt; ホストネットワーク」を開いて
HostNet Driver: tap になっていれば動いてるはずです。
<q>Run nono,
and open "Monitor &gt; Host &gt; Host Network" window from menu.
It's OK if you can see "HostNet Driver: tap".</q>

</ol>

</div>


<a name="changes"></a>
<h4>変更履歴 <q>Changes</q></h4>
<div class="main">
See <a href="changes.html">changes.html</a>.
</div>


<a name="license"></a>
<h4>ライセンス <q>License</q></h4>
<div class="main">
See <a href="nono-license.txt">nono-license.txt</a>.
</div>


<h4>連絡先 <q>Contact us</q></h4>
<div class="main">
バグ報告などは以下にお願いします。日本語でおk。
<q>If you find any problems, please let me know.
You may write in English.</q><br>
<a href="https://github.com/isaki68k/nono-issue/issues"
>https://github.com/isaki68k/nono-issue/issues</a>
</div>


<h4>パッチの提供について <q>About contributes</q></h4>
<div class="main">
パッチを提供してくださる場合は以下に同意したものとします。
<q>If you provide a patch to nono, you must agree to the following conditions:
</q>
<ul>
<li>成果物が nono のライセンスに従って運用あるいは配布されること。
<q>All your work are operated or distributed under the nono license.</q>
<li>ライセンスが将来変わる可能性があること。
<q>The license may be changed in the future.</q>
<li>著作部分に関して著作者人格権を行使しないこと。
<q>Do not exercise your author's rights.</q>
</ul>
<ul>
</div>


<h4>Acknowledgements</h4>
<div class="main">
nono は以下の広告条項を含むソースコードを利用しています。
<q>nono uses source code with the following advertising clause.</q>
<blockquote>
This product includes software developed by Gordon Ross<br>
This product includes software developed by the University of California, Lawrence Berkeley Laboratory.<br>
</blockquote>
</div>


<hr>
nono project
</body>
</html>
