//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "object.h"
#include "logger.h"
#include "mpu.h"
#include "textscreen.h"

// 全オブジェクトのリスト
std::vector<Object*> gObjects;

//
// オブジェクトクラス
//

// コンストラクタ
Object::Object(const std::string& objname_)
{
	// オブジェクト名(ログ表示名)を設定。
	objname = objname_;

	// コンストラクタで指定した場合はエイリアスにも登録しておく。
	AddAlias(objname_);

	// 自分自身をリストに繋ぐ
	gObjects.push_back(this);
}

// デストラクタ
Object::~Object()
{
	// 自分自身をリストから削除する
	// XXX マルチスレッドな状況からは呼ばれないはず?
	for (auto it = gObjects.begin(); it != gObjects.end(); ++it) {
		if (*it == this) {
			gObjects.erase(it);
			break;
		}
	}
}

// メッセージ表示
// 常に表示 (あらかじめ loglevel の評価をした後で呼び出す用)
void
Object::putmsgn(const char *fmt, ...) const
{
	char buf[1024];
	int len;
	va_list ap;

	// 名前だけ生表示されても分かりづらいので括弧でもつけておく
	len = snprintf(buf, sizeof(buf), "(%s) ", GetName().c_str());
	va_start(ap, fmt);
	vsnprintf(buf + len, sizeof(buf) - len, fmt, ap);
	va_end(ap);

	gLogger.Write(buf);
}

// ログ出力 (可変引数)
// 常に表示 (あらかじめ loglevel の評価をした後で呼び出す用)
void
Object::putlogn(const char *fmt, ...) const
{
	char buf[1024];
	va_list ap;
	uint64 vt;
	int len = 0;

	vt = gMPU->GetVirtTime();
	len += snprintf(buf + len, sizeof(buf) - len, "%4d.%03d'%03d'%03d ",
		(int)(vt / 1000 / 1000 / 1000),
		(int)((vt / 1000 / 1000) % 1000),
		(int)((vt / 1000) % 1000),
		(int)(vt % 1000));

	len += snprintf(buf + len, sizeof(buf) - len, "%08x %s ",
		gMPU->GetPPC(), GetName().c_str());

	va_start(ap, fmt);
	vsnprintf(buf + len, sizeof(buf) - len, fmt, ap);
	va_end(ap);

	gLogger.Write(buf);
}
