//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "event.h"
#include "scheduler.h"

// 全イベントの配列
std::vector<Event *> gEvents;

// コンストラクタ
Event::Event()
{
	// 自身をリストに追加する
	gEvents.push_back(this);
}

// コンストラクタ (dev 指定あり)
Event::Event(Device *dev_)
	: Event()
{
	dev = dev_;
}

// デストラクタ
Event::~Event()
{
}

// イベントタイマーを開始する
void
Event::Start()
{
	gScheduler->StartEvent(this);
}

// イベントタイマーを停止する
void
Event::Stop()
{
	gScheduler->StopEvent(this);
}

// 名前をセットする
void
Event::SetName(const std::string& val)
{
	name = val;
	if (name.length() > 25) {
		dev->putmsgn("Event Name \"%s\" is too long(%d chars)",
			name.c_str(), (int)name.length());
		name.resize(25);
	}
}
