//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

// MPU 共通部

#include "mpu.h"
#include "config.h"

std::unique_ptr<MPUDevice> gMPU;

// コンストラクタ
MPUDevice::MPUDevice(const std::string& objname_)
	: inherited(objname_)
{
	ClearAlias();
	AddAlias("MPU");
}

// デストラクタ
MPUDevice::~MPUDevice()
{
}

bool
MPUDevice::Init()
{
	// MPU クロック
	const ConfigItem& item = gConfig->Find("mpu-clock");
	const std::string& val = item.AsString();
	if (val.empty()) {
		item.Err();
		return false;
	}
	// 文字列を double に変換
	char *end;
	errno = 0;
	double f = strtod(val.c_str(), &end);
	if (end == val.c_str() || end[0] != '\0' || errno == ERANGE) {
		item.Err();
		return false;
	}
	// 設定の "mpu-clock" は MHz 単位だが、変数 clock_khz は kHz 単位。
	clock_khz = (int)(f * 1000);
	// 1MHz 未満はエラー。実際 10MHz 未満でもいい気がするけど
	if (clock_khz < 1000) {
		item.Err();
		return false;
	}

	return true;
}
