//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"
#include "mappedfile.h"

class ROMDevice : public IODevice
{
	using inherited = IODevice;
 public:
	ROMDevice(const std::string& objname_);
	virtual ~ROMDevice() override;

	void ResetHard() override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

	// アクセスウェイトを設定
	void SetWait(uint32 wait_) { wait = wait_; }

 protected:
	bool LoadROM(const char *name, int size);

	// アドレスマスク
	uint32 mask {};

	// オープンしているファイル
	std::string filename {};
	MappedFile file {};

	// メモリ領域をさすポインタ。
	// 通常は ROM イメージファイルを読み込むだけなので
	// mem = file.GetPtr() となる。
	// ROM イメージを自分で用意したい場合はこれを適当なところにむけてどうぞ。
	uint8 *mem {};

	// アクセスウェイト
	uint32 wait {};
};
