//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "vm.h"
#include "human68k.h"
#include "xiospace.h"

class VM_X68030 final : public VM
{
	using inherited = VM;
 public:
	VM_X68030();
	~VM_X68030() override;

	// 電源ボタンを押す
	void PowerButton() override;

	// 電源ボタンの状態を取得。オンなら true。
	bool IsPowerButton() const override { return power_button; }

	// MPU の RESET 命令によるリセット
	void ResetByMPU() override;

	// ブートページを切り替える。
	void SwitchBootPage(Device *parent, bool isrom) override;

 private:
	// 電源ボタンの状態。オンなら true。
	bool power_button {};
};

class VM_RXZ final : public VM
{
	using inherited = VM;
 public:
	VM_RXZ();
	~VM_RXZ() override;

	// 電源ボタンを押す。
	void PowerButton() override;

	// ブートページを切り替える。
	void SwitchBootPage(Device *parent, bool isrom) override;

 private:
	std::unique_ptr<Human68k> human68k {};
};
