//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"
#include <array>

class XIOSpaceDevice : public IODevice
{
	using inherited = IODevice;
 public:
	XIOSpaceDevice();
	virtual ~XIOSpaceDevice() override;

	bool Init() override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

	// ブートページを切り替える。
	void SwitchBootPage(bool isrom);

 private:
	// アドレス addr に対応する IODevice* を返す。
	// addr は 24bit マスクされていること。
	inline IODevice *SearchDevice(uint32 addr) const
	{
		uint idx = addr / 8192;
		__assume(idx < 2048);
		return table[idx];
	}

	void AddDevice(IODevice *dev);

	std::array<IODevice*, 2048> table {};
};

extern std::unique_ptr<XIOSpaceDevice> gXIOSpace;
