//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "wxsubwindow.h"
#include "monitor.h"

//
// メモリダンプウィンドウ
//
class WXMemdumpWindow : public WXVResizeSubWindow
{
	using inherited = WXVResizeSubWindow;
 public:
	WXMemdumpWindow(wxWindow *parent, int idx_);
	virtual ~WXMemdumpWindow() override;

	// ウィンドウID(モニタID) に対応する sticky address を取得する
	static uint32 GetStickyAddr(int id);
	// ウィンドウID(モニタID) に対応する sitcky address を設定する
	static void SetStickyAddr(int id, uint32 addr);

 private:
	// ウィンドウサイズ再計算
	void DoSize() override;

	// テキスト入力イベント
	void OnTextEnter(wxCommandEvent& event);

	// ページアップダウンイベント
	void OnPrev(wxCommandEvent& event);
	void OnNext(wxCommandEvent& event);

	// アドレス欄を更新
	void UpdateAddr();

	// コントロール
	wxTextCtrl *addrctrl {};
	wxButton *prev_btn {};
	wxButton *next_btn {};

	// コントロールパネルを格納する sizer
	wxBoxSizer *ctrlbox {};

	// 識別子、または sticky_addr[] の添字。
	// メモリダンプウィンドウは複数枚あるので自分がそのうちの何番目か。
	int idx {};

	// イベントテーブル
	wxDECLARE_EVENT_TABLE();

	// 開始番地の二次記憶域。
	// サブウィンドウは今のところ開いてる時だけオブジェクトが存在している
	// タイプだが、メモリダンプウィンドウを閉じてまた開くとさっきのアドレスが
	// 見えていてほしいので、ここでその状況を真似する。
	// こっちはあくまで補助用の領域で、現在表示中のアドレスの一次情報は
	// screen.userdata のほう。
	static std::array<uint32, MAX_MEMDUMP_MONITOR> sticky_addr;
};
