//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "wxtextpanel.h"
#include "wxbitmapbuf.h"
#include "monitor.h"

//
// テキストスクリーンを表示するパネル
//
// wxPanel
//  ↓
// WXTextPanel  テキストフォントを管理するクラス
//  ↓
// WXTextScreen テキストスクリーンを表示するパネル
//
class WXTextScreen : public WXTextPanel
{
	using inherited = WXTextPanel;

 public:
	WXTextScreen(wxWindow *parent, Monitor& monitor);
	virtual ~WXTextScreen() override;

	// 画面を更新して再描画する。
	void DoRefresh();

	// フォントサイズ変更
	void SetFontSize(FontId fontid) override;

	// 更新頻度設定
	void SetRate(int hz);

	// 桁数、行数を取得する
	int GetCol() const { return ts.GetCol(); }
	int GetRow() const { return ts.GetRow(); }

	// TextScreen のユーザデータ取得、設定
	uint64 GetUserData() const { return ts.userdata; }
	void SetUserData(uint64 value) { ts.userdata = value; }

 private:
	void OnPaint(wxPaintEvent&);
	void OnSize(wxSizeEvent&);
	void OnTimer(wxTimerEvent&);

	void Init(int col, int row);
	void Draw(wxDC& dc);

	// コントロール全域のビットマップ
	WXBitmapBuf bitmap {};

	std::vector<uint16> prevbuf {};

	Monitor& monitor;
	TextScreen ts {};

	wxTimer timer {};

	wxDECLARE_EVENT_TABLE();
};
